@@ -3,9 +3,9 @@
 #' @inheritParams docdb_create
 #'
 #' @param limit (integer) Maximum number of documents
-#'  to return (defaults 10,000 for Elasticsearch and
-#'  all for MongoDB, SQLite, CouchDB, PostgreSQL, and
-#'  DuckDB
+#'  to return. If not set, defaults to 10,000 for
+#'  Elasticsearch and all documents for MongoDB,
+#'  SQLite, CouchDB, PostgreSQL, and DuckDB.
 #'
 #' @param ... Passed on to functions:
 #' - MongoDB: find() in [mongolite::mongo()]
@@ -63,7 +63,7 @@ docdb_get.src_elastic <- function(src, key, limit = "10000", ...) {
     size = limit, ...)[["hits"]][["hits"]]
 
   # early exit
-  if(!length(docids)) return(NULL)
+  if (!length(docids)) return(NULL)
 
   # get ids
   docids <- sapply(docids, "[[", "_id", USE.NAMES = FALSE, simplify = TRUE)
@@ -119,49 +119,41 @@ docdb_get.src_mongo <- function(src, key, limit = NULL, ...) {
 #' @export
 docdb_get.src_sqlite <- function(src, key, limit = NULL, ...) {
 
-  statement <- paste0(
-    "SELECT '{\"_id\": \"' || _id || '\", ' || LTRIM(json, '{') ",
-    "AS json FROM \"", key, "\" ",
-    # canonical sorting in nodbi
-    "ORDER BY _id ASC;")
-
-  return(sqlGet(src = src, key = key, limit = limit, statement = statement, ...))
+  getFunction <- "json"
+  return(sqlGet(src = src, key = key, limit = limit, getFunction = getFunction, ...))
 }
 
 #' @export
 docdb_get.src_postgres <- function(src, key, limit = NULL, ...) {
 
-  statement <- paste0(
-    "SELECT '{\"_id\": \"' || _id || '\", ' || LTRIM(json::TEXT, '{') ",
-    "AS json FROM \"", key, "\" ",
-    # canonical sorting in nodbi
-    "ORDER BY _id ASC;")
-
-  return(sqlGet(src = src, key = key, limit = limit, statement = statement, ...))
+  getFunction <- "json::TEXT"
+  return(sqlGet(src = src, key = key, limit = limit, getFunction = getFunction, ...))
 }
 
 #' @export
 docdb_get.src_duckdb <- function(src, key, limit = NULL, ...) {
 
-  statement <- paste0(
-    "SELECT '{\"_id\": \"' || _id || '\", ' || LTRIM(json, '{') ",
-    "AS json FROM \"", key, "\" ",
-    # canonical sorting in nodbi
-    "ORDER BY _id ASC;")
-
-  return(sqlGet(src = src, key = key, limit = limit, statement = statement, ...))
+  getFunction <- "json"
+  return(sqlGet(src = src, key = key, limit = limit, getFunction = getFunction, ...))
 }
 
 ## helpers --------------------------------------
 
 #' @keywords internal
 #' @noRd
-sqlGet <- function(src, key, limit = NULL, statement, ...) {
+sqlGet <- function(src, key, limit = NULL, getFunction, ...) {
 
   # set limit if not null
   n <- -1L
   if (!is.null(limit)) n <- limit
 
+  # compose query statment
+  statement <- paste0(
+    "SELECT '{\"_id\": \"' || _id || '\", ' || LTRIM(", getFunction, ", '{') ",
+    "AS json FROM \"", key, "\" WHERE json != '{}' ",
+    # canonical sorting in nodbi
+    "ORDER BY _id ASC;")
+
   # temporary file for streaming
   tfname <- tempfile()
   tfnameCon <- file(description = tfname, open = "wt", encoding = "native.enc")
@@ -174,16 +166,18 @@ sqlGet <- function(src, key, limit = NULL, statement, ...) {
     paste0(
       # protect against empty query result
       "",
-      DBI::dbGetQuery(
+     # eliminate rows without any json
+      stats::na.omit(
+        DBI::dbGetQuery(
         conn = src$con,
         statement = statement,
-        n = n)[["json"]]),
+        n = n)[["json"]])),
     con = tfnameCon,
     sep = "\n",
     useBytes = TRUE)
   close(tfnameCon)
 
   # stream in ndjson records
-  return(jsonlite::stream_in(file(tfname, encoding = "UTF-8"), verbose = FALSE))
+  return(jsonlite::stream_in(file(tfname), verbose = FALSE))
 
 }
